/* Try to get the Si5351 Calibrator running on an encoder and OLED.
 */

#define Version "Si5351CalibratorOLED"

#include "si5351.h"
#include "Wire.h"
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 32 // OLED display height, in pixels
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT);

int32_t cal_factor = 0;
int32_t old_cal = 0;
byte updateFlag=HIGH;

uint64_t rx_freq;
uint64_t target_freq = 1000000000ULL; // 10 MHz, in hundredths of hertz

int  temp = 0;              // temperary scratch pad
int loops = 0;
unsigned long step_size =     1000;     // default tuning increment (Hz)

long encoder;                               // encoder direction: CCW=-1; NC=0; CW=1
Si5351 si5351;                            // initalize clockgen

// encoder definitions
#define ENCODER_B  A0
#define ENCODER_A  A1                       
#define ENCODER_INPUT_PORT PINC
#define ENCODER_BUTTON 5                  // selects tuning decade

void setup() { 

  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.clearDisplay();
  display.display();                  // Update screen
  display.setTextSize(1);             // Normal 1:1 pixel scale
  display.setTextColor(WHITE,BLACK);  // Draw white text
  display.setCursor(0,0);             // Start at top-left corner
  display.println(F(Version));
  display.display(); // Update screen
  delay(1000);
  display.clearDisplay();
  display.setCursor(0,0);             // Start at top-left corner
  display.println("Target = 10Mhz");
  display.display(); // Update screen

  // Start serial and initialize the Si5351
  Serial.begin(57600);
  si5351.init(SI5351_CRYSTAL_LOAD_8PF, 0, 0);
  // Start on target frequency
  si5351.set_correction(cal_factor, SI5351_PLL_INPUT_XO);
  si5351.set_pll(SI5351_PLL_FIXED, SI5351_PLLA);
  si5351.set_freq(target_freq, SI5351_CLK0);
  
  // configure rotary encoder & push-button
  pinMode(ENCODER_A, INPUT_PULLUP);     // encoder leads
  pinMode(ENCODER_B, INPUT_PULLUP);  
  pinMode(ENCODER_BUTTON, INPUT_PULLUP);
  
  rx_freq = target_freq;
  cal_factor = old_cal;

  updateFlag=HIGH;
 
}

/******************************
 main loop
*******************************/
void loop() {
 
  // check the frequency......
  check_step_size();
  encoder = read_encoder();                       // returns +1/0/-1 (cw/nc/ccw)
  if (encoder != 0) {
    rx_freq = rx_freq + (encoder * step_size); 
    updateFlag=HIGH;
  } 
 
  if     (updateFlag==HIGH)
  { cal_factor = (int32_t)(target_freq - rx_freq) + old_cal;
    si5351.set_correction(cal_factor, SI5351_PLL_INPUT_XO);
    si5351.set_pll(SI5351_PLL_FIXED, SI5351_PLLA);
    si5351.pll_reset(SI5351_PLLA);
    si5351.set_freq(target_freq, SI5351_CLK0);
    Serial.print(F("Current difference:"));
    Serial.println(cal_factor);
    
    display.setCursor(0,10);
    display.setTextSize(1);
    display.print("Cal=");
    display.setTextSize(2);
    display.setCursor(30,10);
    display.println("        ");
    display.setCursor(30,10);
    display.println(cal_factor);
    display.display(); // Update screen
    display_freq_step();
    updateFlag = LOW;
  }
}

/******************************
 read encoder direction
*******************************/
char read_encoder() {
  // definitions
  #define DELAY 100                               // number of polling-loops   
  static unsigned short integrator;               // encoder debounce
 
  /* the array values are determined by combining the previous and current A1,A0 
  Gray Code bit-patterns to form an array-index. +1 is assigned to all clockwise 
  (CW) indexes; -1 is assigned to all counter-clockwise (CCW) indexes; and 0 is 
  assigned to the remaining positions to indicate no movement*/   
  static char encoder[] = {0,-1,1,0,1,0,0,-1,-1,0,0,1,0,1,-1,0}; // SWAPPED PHASES
  //   static char encoder[] = {0,1,-1,0,-1,0,0,1,1,0,0,-1,0,-1,1,0};
  static unsigned char encoder_index = 0;
  
  static long count = 0;                         // CW rotation = +1; CCW rotation = -1
  static long this_reading;
  static long last_reading;
  char value;                                    // return value

  /* check for encoder rotation. The absolute "count" value changes as the encoder rotates.
  Typically there are four states (counts) per click, but often more due to contact bounce.
  Bounce doesn't matter so long as the "count" decreases" for CCW rotation; remains constant 
  when there is no rotation; and increases with CW rotation. Bounce can be completely 
  eliminated if we look for the "detent" code and integrate as shown below. */  
  encoder_index <<= 2;                           // shift previous bit-pattern left two places
  encoder_index |= (ENCODER_INPUT_PORT & 0x03);  // get current bit-pattern and combine with previous
  count += encoder[( encoder_index & 0x0f )];     
  
  // debounce encoder
  if ((ENCODER_INPUT_PORT & 0x03) == 3) {        // encoder on an indent (3 decimal == 00000011 binary)
    if (integrator < DELAY) {                    
      integrator++;
    } 
  } else {                                       // encoder not resting on indent
     if (integrator > 0) {
       integrator--;
     }
  }
  
  // prepare the "return" value
  if (integrator >= DELAY) {                     // encoder deemed to be stationary
    this_reading = count;
    if (this_reading < last_reading) value = -1;
    if (this_reading == last_reading) value = 0;
    if (this_reading > last_reading) value = 1;
    last_reading = this_reading;                 // both readings now hold current "count"
    integrator = 0;                              // reset integrator
  } else {
    value = 0;
  }
  return(value);
}
  

/******************************
 check the step size
*******************************/
void check_step_size() {
  // delay definitions
  #define DELAY 30                             // DELAY = debounce(mS)*loops/second
  #define LONG_PUSH 50                        // number of loops considered "long"
  
  // variables
  static boolean transition = false;             // valid during button push
  static unsigned short integrator;              // varies from 0 to MAXIMUM
  static unsigned short push_duration;           // determines step size/direction
  
  // look for first high-to-low transition
  if (!transition && (digitalRead(ENCODER_BUTTON) == LOW)) {
    transition = true;
    integrator = 0;
    push_duration = 0;
  }
  
  // look for last low-to-high transition
  if (transition) {
    if (push_duration < (LONG_PUSH*2)) {         // track push duration
      push_duration++;
    }
    if (digitalRead(ENCODER_BUTTON) == HIGH) {   // ENCODER_BUTTON is HIGH
      if (integrator < DELAY) {        
        integrator++;
      } else {                                   // ENCODER_BUTTON is LOW
        if (integrator > 0) {
    integrator--;
  }
      }
    }
   
    // update the step_size
    if (integrator >= DELAY) { 
      transition = false;
      if (push_duration > LONG_PUSH) {          // LONG PUSH: decrease step size
        step_size = step_size/10;
        if (step_size < 10) {                    //
          step_size = 100000;                   // wrap around    
        }        
      } else {                                  // SHORT PUSH: increase step size
        step_size = step_size*10;
        if (step_size > 100000) {
          step_size = 10;                       // wrap around 
        }         
      }
      
      display_freq_step();                      // update step-size     
    }
  }
}

void display_freq_step() {
    display.setCursor(0,20);
    display.setTextSize(1);
    display.print("    ");
    display.setCursor(0,20);
  switch (step_size) {
        case 10:
       display.print("10");
       break;
        case 100:
       display.print("100");
       break;
        case 1000:
       display.print("1K");
       break;
        case 10000:
       display.print("10K");
       break;
        case 100000 :
       display.print("100K");
       break;
  } 
  display.display();   
}
 
